<?php

namespace Drupal\xray_audit\Plugin\xray_audit\tasks\SiteStructure;

use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\xray_audit\Plugin\XrayAuditTaskPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Plugin implementation of queries_data_node.
 *
 * @XrayAuditTaskPlugin (
 *   id = "navigation_architecture",
 *   label = @Translation("Navigation Architecture"),
 *   description = @Translation("Navigation Architecture."),
 *   group = "site_structure",
 *   sort = 1,
 *   operations = {
 *      "menu" = {
 *          "label" = "Menus",
 *          "description" = "Menu structures.",
 *          "dependencies" = {"menu_link_content"},
 *          "download" = TRUE,
 *       }
 *   }
 * )
 */
final class XrayAuditNavigationArchitecturePlugin extends XrayAuditTaskPluginBase {

  /**
   * Service "navigation_architecture".
   *
   * @var \Drupal\xray_audit\Services\NavigationArchitectureInterface
   */
  protected $navigationArchitecture;

  /**
   * Service "xray_audit.plugin_repository".
   *
   * @var \Drupal\xray_audit\Services\PluginRepositoryInterface
   */
  protected $pluginRepository;

  /**
   * Service "request_stack".
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Service "form_builder".
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->navigationArchitecture = $container->get('xray_audit.navigation_architecture');
    $instance->pluginRepository = $container->get('xray_audit.plugin_repository');
    $instance->requestStack = $container->get('request_stack');
    $instance->formBuilder = $container->get('form_builder');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function getDataOperationResult(string $operation = ''): array {
    switch ($operation) {
      case 'menu':
        $query_parameters_values = $this->getQueryParametersFromRequest();
        $show_parent_reference = !empty($query_parameters_values['show_parent_reference']);

        $all_menus_items = [];
        $overall_max_depth = 0;
        $total_items = 0;

        $menu_list = $this->navigationArchitecture->getMenuList();

        if (empty($menu_list)) {
          return [
            'all_items' => [],
            'level_max_overall' => 0,
            'total_items_overall' => 0,
            'show_parent_reference' => $show_parent_reference,
          ];
        }

        foreach (array_keys($menu_list) as $menu_name) {
          $menu_architecture = $this->navigationArchitecture->getMenuArchitecture($menu_name);
          $current_menu_items = $menu_architecture['items'] ?? [];
          $current_menu_level_max = $menu_architecture['level_max'] ?? 0;

          if ($current_menu_level_max > $overall_max_depth) {
            $overall_max_depth = $current_menu_level_max;
          }

          foreach ($current_menu_items as $item) {
            // Tag item with its menu name.
            $item['menu_name'] = $menu_name;
            $all_menus_items[] = $item;
            $total_items++;
          }
        }

        // Sort all items by menu name, then by their original order (implicit if not re-sorting here)
        // Sort by menu_name then by title :
        usort($all_menus_items, function ($a, $b) {
          $menu_compare = strcmp($a['menu_name'], $b['menu_name']);
          if ($menu_compare !== 0) {
            return $menu_compare;
          }
          // Add secondary sort criteria if necessary, e.g., item title or weight.
          return strcmp($a['title'] ?? '', $b['title'] ?? '');
        });

        return [
          'all_items' => $all_menus_items,
          'level_max_overall' => $overall_max_depth,
          'total_items_overall' => $total_items,
          'show_parent_reference' => $show_parent_reference,
        ];
    }

    return [];
  }

  /**
   * Get data from query parameters.
   *
   * @return mixed[]
   *   Data from query parameters.
   */
  protected function getQueryParametersFromRequest(): array {
    $data = ['menu' => NULL, 'show_parent_reference' => NULL];
    $key_items = array_keys($data);

    /**@var \Symfony\Component\HttpFoundation\Request $request*/
    $request = $this->requestStack->getCurrentRequest();
    if (!$request instanceof Request) {
      return $data;
    }

    foreach ($key_items as $item_key) {
      $data[$item_key] = $request->query->get($item_key, NULL);
    }

    return $data;
  }

  /**
   * {@inheritdoc}
   *
   * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
   * @SuppressWarnings(PHPMD.NPathComplexity)
   * @SuppressWarnings(PHPMD.CyclomaticComplexity)
   */
  public function buildDataRenderArray(array $data, string $operation = '') {
    $build = [];

    if (empty($data['all_items'])) {
      $build['#markup'] = $this->t('No menu items found across all menus.');
      // Still add download button for empty state if operation supports download.
      $this->handleCsv($operation, []);
      $build['download_button'] = $this->createRenderableLink($operation, (string) $this->t('Download CSV'));
      $build['download_button']['#weight'] = 5;
      return $build;
    }

    // $menu_id = $data['menu']; // This is no longer a single menu_id
    $html_headers = $this->getHeaders($operation);
    $rows = [];

    // Prepare rows for HTML table.
    foreach ($data['all_items'] as $item) {
      $row = [];
      // Use menu_name from the item.
      $row['menu_name'] = $item['menu_name'];

      for ($i = 1; $i <= $data['level_max_overall']; $i++) {
        switch (TRUE) {
          case $i < $item['level'] && isset($item['levels'][$i]):
            $row['l_' . $i] = [
              'data' => ($data['show_parent_reference']) ? $item['levels'][$i] : '',
              'class' => ['xray-audit-cell-background-color-gray'],
            ];
            break;

          case $i === $item['level']:
            $row['l_' . $i] = $item['title'];
            break;

          default:
            $row['l_' . $i] = '';
        }
      }

      $row['level_val'] = $item['level'];
      $row['enabled_val'] = ($item['enabled']) ? $this->t('Yes') : $this->t('No');
      $row['link_obj'] = !empty($item['link']) ? Link::fromTextAndUrl($item['link']->toString(), $item['link']) : '';
      $row['target_val'] = empty($item['target']) ? '' : implode(' ', $item['target']);

      $rows[] = $row;
    }

    $build['main_table'] = [
      '#theme' => 'table',
    // Use headers from getHeaders.
      '#header' => $html_headers,
      '#rows' => $rows,
      '#weight' => 30,
      '#attributes' => [
        'class' => [
          'xray-audit__table',
          'xray-audit__sticky-header',
          'alignment-vertical-center',
        ],
      ],
      '#attached' => [
        'library' => [
          'xray_audit/xray_audit',
        ],
      ],
    ];

    // CSV Handling using the trait.
    $this->handleCsv($operation, $data);
    $build['download_button'] = $this->createRenderableLink($operation, (string) $this->t('Download CSV'));
    // Ensure it appears before the table or appropriately.
    $build['download_button']['#weight'] = 5;

    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getHeaders(string $operation = ''): array {
    $headers = [];
    $headers['menu_name'] = $this->t('Menu');
    $headers['level_name'] = $this->t('Menu Name');
    $headers['level_val'] = $this->t('Level');
    $headers['enabled_val'] = $this->t('Enabled');
    $headers['link_obj'] = $this->t('Link');
    $headers['target_val'] = $this->t('Link description');
    return $headers;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareCsvHeaders(string $operation = ''): array {
    $data = $this->getDataOperationResult($operation);
    $csv_headers = [];
    $csv_headers[] = $this->t('Menu')->render();

    if (isset($data['level_max_overall'])) {
      for ($i = 1; $i <= $data['level_max_overall']; $i++) {
        $csv_headers[] = $this->t('L@level', ['@level' => $i])->render();
      }
    }

    $csv_headers[] = $this->t('Level')->render();
    $csv_headers[] = $this->t('Enabled')->render();
    $csv_headers[] = $this->t('Link URL')->render();
    $csv_headers[] = $this->t('Link Title')->render();
    // Add Link Description header.
    $csv_headers[] = $this->t('Link Description')->render();
    return $csv_headers;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareCsvData(string $operation, array $data): array {
    $csv_rows = [];

    if (empty($data['all_items'])) {
      return [];
    }

    foreach ($data['all_items'] as $item) {
      $row = [];
      $row[] = $item['menu_name'];

      for ($i = 1; $i <= $data['level_max_overall']; $i++) {
        if ($i < $item['level'] && isset($item['levels'][$i])) {
          $row[] = ($data['show_parent_reference']) ? (string) $item['levels'][$i] : '';
        }
        elseif ($i === $item['level']) {
          $row[] = (string) $item['title'];
        }
        else {
          $row[] = '';
        }
      }

      $row[] = (string) $item['level'];
      $row[] = ($item['enabled']) ? $this->t('Yes')->render() : $this->t('No')->render();
      // Handle string links too.
      $row[] = !empty($item['link']) && $item['link'] instanceof Url ? $item['link']->toString() : (!empty($item['link']) && is_string($item['link']) ? $item['link'] : '');
      // Link title or item title.
      $row[] = !empty($item['link']) && $item['link'] instanceof Link ? $item['link']->getText() : ((string) ($item['title'] ?? ''));
      // Add Link Description data.
      $row[] = empty($item['target']) ? '' : implode(' ', $item['target']);

      $csv_rows[] = $row;
    }
    return $csv_rows;
  }

}
